//------------------------------------------------------------------
#property copyright "mladen"
#property link      "mladenfx@gmail.com"
//------------------------------------------------------------------

#property indicator_chart_window
#property indicator_buffers   9
#property indicator_color1    LimeGreen
#property indicator_color2    LimeGreen
#property indicator_color3    DarkGray
#property indicator_color4    Orange
#property indicator_color5    Orange
#property indicator_color6    LimeGreen
#property indicator_color7    Orange
#property indicator_color8    Orange
#property indicator_color9    MediumOrchid
#property indicator_width6    2
#property indicator_width7    2
#property indicator_width8    2
#property indicator_style1    STYLE_DOT
#property indicator_style2    STYLE_DOT
#property indicator_style3    STYLE_DOT
#property indicator_style4    STYLE_DOT
#property indicator_style5    STYLE_DOT
#property indicator_style9    STYLE_DASH

//
//
//
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average (high+low+open+close)/4
   pr_medianb,    // Average median body (open+close)/2
   pr_tbiased,    // Trend biased price
   pr_haclose,    // Heiken ashi close
   pr_haopen ,    // Heiken ashi open
   pr_hahigh,     // Heiken ashi high
   pr_halow,      // Heiken ashi low
   pr_hamedian,   // Heiken ashi median
   pr_hatypical,  // Heiken ashi typical
   pr_haweighted, // Heiken ashi weighted
   pr_haaverage,  // Heiken ashi average
   pr_hamedianb,  // Heiken ashi median body
   pr_hatbiased   // Heiken ashi trend biased price
};

//
//
//
//
//

extern int      StochasticLength = 50;
extern int      SmoothEMA        = 9;
extern int      SignalEMA        = 5;
extern int      AmaPeriod        = 10;
extern enPrices AmaPrice         = pr_close;
extern double   FastEnd          = 2;
extern double   SlowEnd          = 30;
extern double   SmoothPower      = 2;
extern double   FilterDeviations = 1.0; 
extern int      FilterPeriod     = 10;
extern bool     JurikFDAdaptive  = true;
extern int      DonchianLength   = 20;
extern double   Level_Up         = 80;
extern double   Level_Dn         = 20;

//
//
//
//
//

double upup[];
double updn[];
double midd[];
double dnup[];
double dndn[];
double value[];
double valueda[];
double valuedb[];
double sigBuffer[];
double dssBuffer[];
double ama[];
double trend[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(12);
   SetIndexBuffer(0,upup);
   SetIndexBuffer(1,updn);
   SetIndexBuffer(2,midd);
   SetIndexBuffer(3,dnup);
   SetIndexBuffer(4,dndn);
   SetIndexBuffer(5,value);
   SetIndexBuffer(6,valueda);
   SetIndexBuffer(7,valuedb);
   SetIndexBuffer(8,sigBuffer);
   SetIndexBuffer(9,dssBuffer);
   SetIndexBuffer(10,ama);
   SetIndexBuffer(11,trend);
      StochasticLength = MathMax(1,StochasticLength);
      SignalEMA        = MathMax(0,SignalEMA);
      SmoothEMA        = MathMax(0,SmoothEMA);

   //
   //
   //
   //
   //

   IndicatorShortName("DSS of Advanced Kaufman AMA ("+StochasticLength+","+SmoothEMA+","+SignalEMA+","+AmaPeriod+")");
   return(0);
}
int deinit(){ return(0); }

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int k,counted_bars = IndicatorCounted();
   if (counted_bars < 0) return(-1);
   if (counted_bars > 0) counted_bars--;
           int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
   
      double alpha = 2.0 / (1.0+SignalEMA);
      if (trend[Bars-limit-1]==-1) CleanPoint(limit,valueda,valuedb);
   	for(int i = limit; i >= 0; i--)
      {
            ama[i] = iKama(AmaPeriod,FastEnd,SlowEnd,AmaPrice,SmoothPower,JurikFDAdaptive,i);
            
            //
            //
            //
            //
            //
            
            double stddev = 0;
               if (FilterDeviations>0)
               {
                  double avgdiff = 0;
                     for (k=0; k<FilterPeriod; k++) avgdiff += MathAbs(ama[i+k]-ama[i+k+1]);                    avgdiff /= FilterPeriod;
                     for (k=0; k<FilterPeriod; k++) stddev  += MathPow(MathAbs(ama[i+k]-ama[i+k+1])-avgdiff,2); stddev  /= FilterPeriod;
               }
            double filter = FilterDeviations * MathPow(stddev,0.5);
            
            //
            //
            //
            //
            //
            
            dssBuffer[i] = iDss(ama[i],ama[i],ama[i],StochasticLength,SmoothEMA,i);
            upup[i]      = High[iHighest(NULL,0,MODE_HIGH,DonchianLength,i+1)];
            dndn[i]      =  Low[iLowest(NULL,0, MODE_LOW, DonchianLength,i+1)];
            midd[i]      = (upup[i]+dndn[i])/2;
            double range =  upup[i]-midd[i];
            updn[i]      = dndn[i]+range*2.0*Level_Up/100.0;
            dnup[i]      = dndn[i]+range*2.0*Level_Dn/100.0;
            value[i]     = midd[i]+(dssBuffer[i]-50)/100*2*range;
            valueda[i]   = EMPTY_VALUE;
            valuedb[i]   = EMPTY_VALUE;
            trend[i] = trend[i+1];
              if (value[i]>value[i+1]) trend[i] =  1;
              if (value[i]<value[i+1]) trend[i] = -1;
              if (trend[i]==-1) PlotPoint(i,valueda,valuedb,value);
              sigBuffer[i] = sigBuffer[i+1]+alpha*(value[i]-sigBuffer[i+1]);
            
            
                           
      }
   return(0);           
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workDss[][5];
#define _plow  0
#define _phigh 1
#define _st1   2
#define _ss1   3
#define _dss   4

double iDss(double pClose, double pHigh, double pLow, int period, int smoothPeriod, int i, int instanceNo=0)
{
   if (ArrayRange(workDss,0)!=Bars) ArrayResize(workDss,Bars); i = Bars-i-1; instanceNo*=5;

   //
   //
   //
   //
   //
   
         double alpha = 2.0 / (1.0+smoothPeriod);
            workDss[i][instanceNo+_phigh] = pHigh;
            workDss[i][instanceNo+_plow]  = pLow;
               double min   = workDss[i][instanceNo+_plow];
               double max   = workDss[i][instanceNo+_phigh];

      
         for (int k=1; k<period && (i-k)>=0; k++)
         {
            min = MathMin(min,workDss[i-k][instanceNo+_plow] );
            max = MathMax(max,workDss[i-k][instanceNo+_phigh]);
         }
         workDss[i][instanceNo+_st1] = 0; if (min!=max) workDss[i][instanceNo+_st1] = 100*(pClose-min)/(max-min);
         
         //
         //
         //
         //
         //
         
         if (i==0)
         {
               workDss[i][instanceNo+_ss1] = workDss[i][instanceNo+_st1];
               workDss[i][instanceNo+_dss] = workDss[i][instanceNo+_st1];
         }            
         else
         {
            workDss[i][instanceNo+_ss1] = workDss[i-1][instanceNo+_ss1]+alpha*(workDss[i][instanceNo+_st1]-workDss[i-1][instanceNo+_ss1]);
               min = workDss[i][instanceNo+_ss1];
               max = workDss[i][instanceNo+_ss1];
               for (k=1; k<period && (i-k)>=0; k++)
               {
                  min = MathMin(min,workDss[i-k][instanceNo+_ss1]);
                  max = MathMax(max,workDss[i-k][instanceNo+_ss1]);
               }
               double stoch = 0; if (min!=max) stoch = 100*(workDss[i][instanceNo+_ss1]-min)/(max-min);
               workDss[i][instanceNo+_dss] = workDss[i-1][instanceNo+_dss]+alpha*(stoch-workDss[i-1][instanceNo+_dss]);
         }            
            
      //
      //
      //
      //
      //
            
   return(workDss[i][instanceNo+_dss]);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//

double workAma[][2];
#define _diff 0
#define _kama 1

//
//
//
//
//

double iKama(int period, double fast, double slow, int priceType, double power, bool JurikFD, int i)
{
   if (ArrayRange(workAma,0)!=Bars) ArrayResize(workAma,Bars);

   double price = iMA(NULL,0,1,0,MODE_SMA,priceType,i); int r = Bars-i-1; if (r<period) return(price);
   double fastend = (2.0 /(fast + 1));
   double slowend = (2.0 /(slow + 1));
   
   //
   //
   //
   //
   //

      if (JurikFD) double efratio = MathMin(2.0-iJurikFractalDimension(period,2,i),1.0);
      else
      {
         double signal = MathAbs(price-iMA(NULL,0,1,0,MODE_SMA,priceType,i+period));
         double noise  = 0;
            workAma[r][_diff] = MathAbs(price-iMA(NULL,0,1,0,MODE_SMA,priceType,i+1));
            for (int k=0; k<period; k++) 
                  noise += workAma[r-k][_diff];

            //
            //
            //
            //
            //

            if (noise != 0)
                  efratio = signal/noise;
            else  efratio = 1;
      }
      double smooth = MathPow(efratio*(fastend-slowend)+slowend,power);
             workAma[r][_kama] = workAma[r-1][_kama] + smooth*(price-workAma[r-1][_kama]);

   //
   //
   //
   //
   //
      
   return(workAma[r][_kama]);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

double workFd[][2];
#define _smlRange 0
#define _smlSumm  1

//
//
//
//
//

double iJurikFractalDimension(int size, int count,int i)
{
   if (ArrayRange(workFd,0)!=Bars) ArrayResize(workFd,Bars);

   int window1  = size*(count-1);
   int window2  = size* count;
   int r        = Bars-i-1;

   //
   //
   //
   //
   //

      workFd[r][_smlRange] = iFdRange(size,i);
      workFd[r][_smlSumm]  = workFd[r-1][_smlSumm]+workFd[r][_smlRange];
      
      if ((Bars-i)>window1)
      {
         workFd[r][_smlSumm] -= workFd[r-window1][_smlRange];
         return(2.0-MathLog(iFdRange(window2,i)/(workFd[r][_smlSumm]/window1))/MathLog(count));
      }
      else return(0.5);
}
double iFdRange(int size,int i) { return(MathMax(Close[size+i],High[ArrayMaximum(High,size,i)])-MathMin(Close[size+i],Low[ArrayMinimum(Low,size,i)])); }   

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//
//

double workHa[][4];
double getPrice(int price, const double& open[], const double& close[], const double& high[], const double& low[], int i, int instanceNo=0)
{
  if (price>=pr_haclose && price<=pr_hatbiased)
   {
      if (ArrayRange(workHa,0)!= Bars) ArrayResize(workHa,Bars);
         int r = Bars-i-1;
         
         //
         //
         //
         //
         //
         
         double haOpen;
         if (r>0)
                haOpen  = (workHa[r-1][instanceNo+2] + workHa[r-1][instanceNo+3])/2.0;
         else   haOpen  = (open[i]+close[i])/2;
         double haClose = (open[i] + high[i] + low[i] + close[i]) / 4.0;
         double haHigh  = MathMax(high[i], MathMax(haOpen,haClose));
         double haLow   = MathMin(low[i] , MathMin(haOpen,haClose));

         if(haOpen  <haClose) { workHa[r][instanceNo+0] = haLow;  workHa[r][instanceNo+1] = haHigh; } 
         else                 { workHa[r][instanceNo+0] = haHigh; workHa[r][instanceNo+1] = haLow;  } 
                                workHa[r][instanceNo+2] = haOpen;
                                workHa[r][instanceNo+3] = haClose;
         //
         //
         //
         //
         //
         
         switch (price)
         {
            case pr_haclose:     return(haClose);
            case pr_haopen:      return(haOpen);
            case pr_hahigh:      return(haHigh);
            case pr_halow:       return(haLow);
            case pr_hamedian:    return((haHigh+haLow)/2.0);
            case pr_hamedianb:   return((haOpen+haClose)/2.0);
            case pr_hatypical:   return((haHigh+haLow+haClose)/3.0);
            case pr_haweighted:  return((haHigh+haLow+haClose+haClose)/4.0);
            case pr_haaverage:   return((haHigh+haLow+haClose+haOpen)/4.0);
            case pr_hatbiased:
               if (haClose>haOpen)
                     return((haHigh+haClose)/2.0);
               else  return((haLow+haClose)/2.0);        
         }
   }
   
   //
   //
   //
   //
   //
   
   switch (price)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_medianb:   return((open[i]+close[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
      case pr_tbiased:   
               if (close[i]>open[i])
                     return((high[i]+close[i])/2.0);
               else  return((low[i]+close[i])/2.0);        
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}
void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
         if (first[i+2] == EMPTY_VALUE) 
              { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
         else { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else       { first[i]  = from[i];                          second[i] = EMPTY_VALUE; }
}
